import sys
import time
import numpy as np
import cv2
from picamera2 import Picamera2
from libcamera import controls
import RPi.GPIO as GPIO
import smbus
#import os

#CAM_WIDTH	= 1920
#CAM_HEIGHT	= 1080

# 64M
#CAM_WIDTH	= 9152
#CAM_HEIGHT	= 6944
#CAM_SHRINK	= 4

# V3
#CAM_WIDTH	= 4608
#CAM_HEIGHT	= 2592
#CAM_SHRINK	= 2

# HQ camera
#CAM_WIDTH	= 4056
#CAM_HEIGHT	= 3040
#CAM_SHRINK	= 2

# V2
CAM_WIDTH	= 3280
CAM_HEIGHT	= 2464
CAM_SHRINK	= 1


WINNAME		= 'camera'
WIN_WIDTH	= int((CAM_WIDTH / CAM_SHRINK) / 2)
WIN_HEIGHT	= int((CAM_HEIGHT / CAM_SHRINK) / 2)

# (mouse)
mouse_state	= 0
mouse_x0	= 0
mouse_y0	= 0
mouse_x1	= 0
mouse_y1	= 0

def mouse_func(event, x, y, flags, param):
	global	mouse_state, mouse_x0, mouse_y0, mouse_x1, mouse_y1
	x_cutsize	= int((CAM_WIDTH / CAM_SHRINK) / 30)
	y_cutsize	= int((CAM_HEIGHT / CAM_SHRINK) / 30)

	if event == cv2.EVENT_LBUTTONDOWN:
		mouse_state	= mouse_state ^ 1
		if mouse_state != 0:
			if x + x_cutsize < CAM_WIDTH / CAM_SHRINK:
				mouse_x0	= int(x)
				mouse_x1	= int(mouse_x0 + x_cutsize - 1)
			else:
				mouse_x1	= int(CAM_WIDTH / CAM_SHRINK - 1)
				mouse_x0	= int(mouse_x1 - x_cutsize)

			if y + y_cutsize < CAM_HEIGHT / CAM_SHRINK:
				mouse_y0	= int(y)
				mouse_y1	= int(mouse_y0 + y_cutsize - 1)
			else:
				mouse_y1	= int(CAM_HEIGHT / CAM_SHRINK - 1)
				mouse_y0	= int(mouse_y1 - y_cutsize)

			# print("m[%d,%d] s[%d,%d] t<%d,%d>-b<%d,%d>" % (x, y, x_cutsize, y_cutsize, mouse_x0, mouse_y0, mouse_x1, mouse_y1) )

# open camera
cap	= Picamera2()
config	= cap.create_still_configuration(main={"size":(int(CAM_WIDTH / CAM_SHRINK), int(CAM_HEIGHT / CAM_SHRINK))})
cap.configure(config)
cap.start()

# display
cv2.namedWindow(WINNAME, cv2.WINDOW_NORMAL)
cv2.resizeWindow(WINNAME, WIN_WIDTH, WIN_HEIGHT)
cv2.setMouseCallback(WINNAME, mouse_func)

# init parameters
ctr_x	= int(CAM_WIDTH / CAM_SHRINK / 2)
ctr_y	= int(CAM_HEIGHT / CAM_SHRINK / 2)
radius	= int(CAM_WIDTH / CAM_SHRINK / 2.5)

# wait key input
while 1:
	frame_rgb	= cap.capture_array()
	frame		= cv2.cvtColor(frame_rgb, cv2.COLOR_RGB2BGR)

	if mouse_state == 0:
		cv2.circle(frame, (ctr_x, ctr_y), radius, (0, 0, 255), 3)
		cv2.imshow(WINNAME, frame)
	else:
		frame_roi	= frame[mouse_y0 : mouse_y1, mouse_x0 : mouse_x1]
		frame_zoom	= cv2.resize(frame_roi, (WIN_WIDTH, WIN_HEIGHT), 0.0, 0.0, cv2.INTER_CUBIC)
		cv2.imshow(WINNAME, frame_zoom)

	key	= cv2.waitKey(1)
	if key == ord('p'):		# enlarge
		radius	= radius + 1
	elif key == ord('m'):	# shrink
		radius	= radius - 1
	elif key == ord('8'):	# up
		ctr_y	= ctr_y - 1;
	elif key == ord('2'):	# down
		ctr_y	= ctr_y + 1;
	elif key == ord('4'):	# left
		ctr_x	= ctr_x - 1;
	elif key == ord('6'):	# right
		ctr_x	= ctr_x + 1;
	elif key == ord('q'):
		print("X = %d, Y = %d, R = %d" % (ctr_x * CAM_SHRINK, ctr_y * CAM_SHRINK, radius * CAM_SHRINK))
		break

cap.stop()
cv2.destroyWindow(WINNAME)
