import sys
import time
import numpy as np
import cv2
from picamera2 import Picamera2
from libcamera import controls
import RPi.GPIO as GPIO
import smbus
#import os

#SCALE	= 2
SCALE	= 1


######################################################################
# resolution

# 64M
#CAM_WIDTH	= 4624
#CAM_HEIGHT	= 3472
#CAM_WIDTH	= int(9152 / SCALE)
#CAM_HEIGHT	= int(6944 / SCALE)

# V3
#CAM_WIDTH	= int(4608 / SCALE)
#CAM_HEIGHT	= int(2592 / SCALE)

# HQ camera
#CAM_WIDTH	= int(4056 / SCALE)
#CAM_HEIGHT	= int(3040 / SCALE)

# V2
#CAM_WIDTH	= int(3280 / SCALE)
#CAM_HEIGHT	= int(2464 / SCALE)

######################################################################
# position

# right (CAM A)
#CAM_R_X	= int(4440 / SCALE)
#CAM_R_Y	= int(3472 / SCALE)
#CAM_R_R	= int(5368 / SCALE)
# left (CAM C)
#CAM_L_X	= int(4824 / SCALE)
#CAM_L_Y	= int(3472 / SCALE)
#CAM_L_R	= int(5292 / SCALE)


######################################################################

WINNAME		= 'camera'
WIN_WIDTH	= int(CAM_WIDTH / 8)
WIN_HEIGHT	= int(CAM_HEIGHT / 8)

# init GPIO, I2C
GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)
GPIO.setup(4, GPIO.OUT)
GPIO.setup(17, GPIO.OUT)
GPIO.setup(18, GPIO.OUT)
i2c	= smbus.SMBus(1)

# camA
def	set_camA():
	i2c.write_byte_data(0x70, 0x00, 0x04)
	GPIO.output(4, 0)
	GPIO.output(17, 0)
	GPIO.output(18, 1)

# camC
def	set_camC():
	i2c.write_byte_data(0x70, 0x00, 0x06)
	GPIO.output(4, 0)
	GPIO.output(17, 1)
	GPIO.output(18, 0)

# open camera
set_camA()

cap	= Picamera2()
config	= cap.create_still_configuration(main={"size":(CAM_WIDTH, CAM_HEIGHT)})
cap.configure(config)

# display
cv2.namedWindow(WINNAME, cv2.WINDOW_NORMAL)
cv2.resizeWindow(WINNAME, WIN_WIDTH, WIN_HEIGHT)

# wait key input
start_time	= time.clock_gettime_ns(time.CLOCK_MONOTONIC)
frame_ctr	= 1

while 1:
	print("A")
	set_camA()
	cap.start()
	frame_r_rgb	= cap.capture_array()
	cap.stop()
	frame_r		= cv2.cvtColor(frame_r_rgb, cv2.COLOR_RGB2BGR)

	print("C")
	set_camC()
	cap.start()
	frame_l_rgb	= cap.capture_array()
	cap.stop()
	frame_l		= cv2.cvtColor(frame_l_rgb, cv2.COLOR_RGB2BGR)

	frame		= cv2.hconcat([frame_l, frame_r])
	cv2.imshow(WINNAME, frame)

	key	= cv2.waitKey(1)
	if key == ord('q'):
		break

	cur_time	= time.clock_gettime_ns(time.CLOCK_MONOTONIC)
	fps			= frame_ctr / ((cur_time - start_time) / 1000000000)
	str			= "%.1f fps\r" % (fps)
	print(str)
	frame_ctr	= frame_ctr + 1

# output image
print("saving output")

# cut image
print(frame_l.shape)
print(frame_r.shape)

l_top		= int(max(CAM_L_Y - CAM_L_R, 0))
l_bottom	= int(min(CAM_L_Y + CAM_L_R, CAM_HEIGHT))
l_left		= int(max(CAM_L_X - CAM_L_R, 0))
l_right		= int(min(CAM_L_X + CAM_L_R, CAM_WIDTH))

r_top		= int(max(CAM_R_Y - CAM_R_R, 0))
r_bottom	= int(min(CAM_R_Y + CAM_R_R, CAM_HEIGHT))
r_left		= int(max(CAM_R_X - CAM_R_R, 0))
r_right		= int(min(CAM_R_X + CAM_R_R, CAM_WIDTH))

frame_l_cut	= frame_l[l_top:l_bottom, l_left:l_right]
frame_r_cut	= frame_r[r_top:r_bottom, r_left:r_right]

print(frame_l_cut.shape)
print(frame_r_cut.shape)

# resize
frame_l_resize	= cv2.resize(frame_l_cut, (int(CAM_L_R * 2), int(CAM_L_R * 2)))
frame_r_resize	= cv2.resize(frame_r_cut, (int(CAM_L_R * 2), int(CAM_L_R * 2)))	# NOTE: use CAM_L

print(frame_l_resize.shape)
print(frame_r_resize.shape)

# concat
#outimg	= cv2.hconcat([frame_l, frame_r])
outimg	= cv2.hconcat([frame_l_resize, frame_r_resize])

print(outimg.shape)

cv2.imwrite('capture.jpg', outimg)
cv2.imwrite('capture.png', outimg)
cv2.imwrite('l.png', frame_l_resize)
cv2.imwrite('r.png', frame_r_resize)

cv2.destroyWindow(WINNAME)
